// ----------------------------------------------------------------------------
//
// VDream Component Suite version 7.0
//
// http://www.gilgil.net
//
// Copyright (c) Gilbert Lee All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef __DEBUG_NEW_H__
#define __DEBUG_NEW_H__

#include <malloc.h>
#include <stdio.h> // for size_t

#define debug_new_log // printf

// ----------------------------------------------------------------------------
// Overload functions
// ----------------------------------------------------------------------------
void* debug_malloc      (size_t size, const char* file, const int line);
void  debug_free        (void* p,     const char* file, const int line);

void* operator new      (size_t size, const char* file, const int line)
{
  void* p = debug_malloc(size, file, line);
  debug_new_log("new(%u, %s, %d) %p\n", size, file, line, p);
  return p;
}

void  operator delete   (void* p, const char* file, const int line)
{
  debug_new_log("delete(%p, %s, %d)\n", p, file, line);
  return debug_free(p, file, line);
}

void operator delete(void* p)
{
  debug_new_log("delete(%p)\n", p);
  return debug_free(p, "unknown", 0);
}

void* operator new[]    (size_t size, const char* file, const int line)
{
  void* p = debug_malloc(size, file, line);
  debug_new_log("new[](%u, %s, %d) %p\n", size, file, line, p);
  return p;
}

void operator delete[](void* p)
{
  debug_new_log("delete[](%p)\n", p);
  return debug_free(p, "unknown", 0);
}

void  operator delete[] (void* p, const char* file, const int line)
{
  debug_new_log("delete[](%p, %s, %d)\n", p, file, line);
  return debug_free(p, file, line);
}

// ----------------------------------------------------------------------------
// Macros
// ----------------------------------------------------------------------------
#define malloc(A) debug_malloc((A), __FILE__, __LINE__)
#define free(A)   debug_free((A), __FILE__, __LINE__)
#define new       new(__FILE__, __LINE__)
#define delete(A) operator delete((A), __FILE__, __LINE__)

#endif // __DEBUG_NEW_H__

