#include "MemoryLeak.h"

#include <malloc.h>

// ----------------------------------------------------------------------------
// MemoryLea
// ----------------------------------------------------------------------------
MemoryLeak g_memoryLeak;

MemoryLeak::MemoryLeak()
{
  start(MEMORY_LEAK_CNT);
  m_del_checking = false;
}

MemoryLeak::~MemoryLeak()
{
  stop(false); // check last memory leak
  start(0);
  m_del_checking = false;
}

void MemoryLeak::_start(int cnt)
{
  if (m_info != NULL)
  {
    ::free(m_info);
    m_info = NULL;
  }
  if (cnt != 0)
  {
    m_info = (MemoryLeakInfo*) ::malloc(sizeof(MemoryLeakInfo) * cnt);
    for (int i = 0; i < cnt; i++)
    {
      MemoryLeakInfo* info = &m_info[i];
      info->p    = NULL;
      info->size = 0;
      info->file = NULL;
      info->line = 0;
    }
  }
  m_cnt = cnt;
  m_del_checking = true;
}

int MemoryLeak::_stop(bool auto_free)
{
  int res = 0;
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p != NULL)
    {
      printf("memory leak %p(%u) %s:%d\n", info->p, info->size, info->file, info->line);
      res++;
      if (auto_free)
      {
        ::free(info->p);
        info->p    = NULL;
        info->size = 0;
        info->file = NULL;
        info->line = 0;
      }
    }
  }
  m_del_checking = false;
  return res;
}

MemoryLeakInfo* MemoryLeak::find(void* p)
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p == p)
    {
      return info;
    }
  }
  return NULL;
}

MemoryLeakInfo* MemoryLeak::add(void* p, size_t size, char* file, int line)
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p == NULL)
    {
      info->p    = p;
      info->size = size;
      info->file = file;
      info->line = line;
      return info;
    }
  }
  return NULL;
}

MemoryLeakInfo* MemoryLeak::del(void* p)
{
  for (int i = 0; i < m_cnt; i++)
  {
    MemoryLeakInfo* info = &m_info[i];
    if (info->p == p)
    {
      info->p    = NULL;
      info->size = 0;
      info->file = NULL;
      info->line = 0;
      return info;
    }
  }
  return NULL;
}

void MemoryLeak::start(int cnt)
{
  g_memoryLeak._start(cnt);
}

int MemoryLeak::stop(bool auto_free)
{
  return g_memoryLeak._stop(auto_free);
}

// ----- gilgil temp 2011.09.05 -----
/*
class AutoInitializer
{
public:
  AutoInitializer()          { }
  virtual ~AutoInitializer() { MemoryLeak::stop(); }
};

AutoInitializer autoInitializer; 
*/
// ----------------------------------
